from typing import final

from cow.web.application import BaseLunaApplicationCtx, LunaApplication as _LunaApplication
from luna3.client import Luna3Session
from luna3.settings import HTTPSettings
from sanic import HTTPResponse

from .configs.configs.services import SettingsLambdaUnit
from .configs.configs.settings.classes import ServiceAddressSettings, ServiceTimeoutsSettings
from .monitoring import MonitoringPoint
from .request import BaseHandlersRequest, BaseStandaloneLambdaRequest


class FakeManager:
    """Fake plugin manager"""

    @classmethod
    async def shutdown(cls):
        """Prevent default luna-application behaviour"""

    @staticmethod
    async def sendEventToPlugins(*args, **kwargs):
        """Prevent default luna-application behaviour"""


class CtxClass(BaseLunaApplicationCtx):
    """Empty context class for lambda application"""

    pluginManager = FakeManager
    lambdaVersion: int = 1
    luna3Session: Luna3Session | None
    userCtx: object | None = None
    lambdaName: str | None = None


async def lambdaVersionMiddleware(request: BaseHandlersRequest | BaseStandaloneLambdaRequest, response: HTTPResponse):
    """
    Monitoring middleware. Generate a request point and error point and send them to app monitoring.

    Args:
        request: request
        response: response
    """
    response.headers["Lambda-Version"] = request.app.ctx.lambdaVersion


def getServiceClientKwargs(addressSetting: ServiceAddressSettings, timeoutSettings: ServiceTimeoutsSettings) -> dict:
    """Get service client kwargs"""
    return {
        "origin": addressSetting.origin,
        "api": addressSetting.apiVersion,
        "totalTimeout": timeoutSettings.totalTimeout,
        "connectTimeout": timeoutSettings.connectTimeout,
        "sockConnectTimeout": timeoutSettings.sockConnectTimeout,
        "sockReadTimeout": timeoutSettings.sockReadTimeout,
        "asyncRequest": True,
    }


class BaseLambdaApp(_LunaApplication):
    """Base for lambda application"""

    @final
    def __init__(self, serviceName: str, lambdaVersion: int, lambdaType: str, userCtxCls: type[object]):
        CtxClass.lambdaVersion = lambdaVersion
        from ..public.handlers.handlers_request import HandlersLambdaRequest
        from ..public.standalone.standalone_request import StandaloneLambdaRequest

        super().__init__(
            apiVersion=1,
            configClass=SettingsLambdaUnit,
            defaultPort=8000,
            ctxClass=CtxClass,
            requestClass=HandlersLambdaRequest if lambdaType == "handlers" else StandaloneLambdaRequest,
        )
        self.response_middleware.append(lambdaVersionMiddleware)
        self.ctx.luna3Session = None
        self.ctx.lambdaType = lambdaType
        self.ctx.userCtx = userCtxCls()
        # hack to override lambda name for monitoring
        self.ctx.lambdaName = serviceName

    @final
    async def shutdown(self):
        if hasattr(self.ctx.userCtx, "onShutdown"):
            return await self.ctx.userCtx.onShutdown()

    @final
    async def initialize(self):
        """Description see :func:`~_LunaApplication.initialize`"""
        # it is not supposed to use plugins by lambda unit
        self.ctx.serviceConfig.activePlugins = []
        await super().initialize()
        self.ctx.luna3Session = Luna3Session(**self.getLuna3Kwargs(self.ctx.serviceConfig))
        if hasattr(self.ctx.userCtx, "onStart"):
            return await self.ctx.userCtx.onStart()
        MonitoringPoint.initialize(serviceName=self.ctx.lambdaName)

    @staticmethod
    def getLuna3Kwargs(config: SettingsLambdaUnit):
        lunaKwargs = {
            "facesSettings": HTTPSettings(
                **getServiceClientKwargs(addressSetting=config.facesAddress, timeoutSettings=config.facesTimeouts),
            ),
            "remoteSDKSettings": HTTPSettings(
                **getServiceClientKwargs(
                    addressSetting=config.remoteSDKAddress, timeoutSettings=config.remoteSDKTimeouts
                ),
            ),
        }
        if config.additionalServicesUsage.lunaImageStore:
            lunaKwargs.update(
                {
                    "faceSampleStoreSettings": HTTPSettings(
                        **getServiceClientKwargs(
                            addressSetting=config.faceSamplesStorage, timeoutSettings=config.faceSamplesStorageTimeouts
                        ),
                    ),
                    "bodySampleStoreSettings": HTTPSettings(
                        **getServiceClientKwargs(
                            addressSetting=config.bodySamplesStorage, timeoutSettings=config.bodySamplesStorageTimeouts
                        ),
                    ),
                    "imageOriginStoreSettings": HTTPSettings(
                        **getServiceClientKwargs(
                            addressSetting=config.imageOriginStorage, timeoutSettings=config.imageOriginStorageTimeouts
                        ),
                    ),
                }
            )
        if config.additionalServicesUsage.lunaEvents:
            lunaKwargs["eventSettings"] = HTTPSettings(
                **getServiceClientKwargs(addressSetting=config.eventsAddress, timeoutSettings=config.eventsTimeouts),
            )
        if config.additionalServicesUsage.lunaMatcherProxy:
            lpmAddress, lpmTimeouts = config.matcherProxyAddress, config.matcherProxyTimeouts
        else:
            lpmAddress, lpmTimeouts = config.pythonMatcherAddress, config.pythonMatcherTimeouts
        lunaKwargs["pythonMatcherSettings"] = HTTPSettings(
            **getServiceClientKwargs(addressSetting=lpmAddress, timeoutSettings=lpmTimeouts)
        )
        if config.additionalServicesUsage.lunaSender:
            lunaKwargs["senderSettings"] = HTTPSettings(
                origin=config.senderAddress.origin,
                api=config.senderAddress.apiVersion,
                asyncRequest=True,
            )
        return lunaKwargs
